<?php

/**
 * Abundant
 * 
 * Originally coded by Martti Laine (@codeclown).
 * 
 * @version 0.1
 * @license MIT
 */

// Error-reporting at full speed for now
error_reporting(E_ALL);
ini_set('display_errors', '1');

// Start a session
session_start();

// Check PHP-version
if(version_compare(phpversion(), '5') < 0) {
	echo 'Abundant requires PHP 5 or newer :)';
	exit;
}

// Define a constant that can be used to verify a file
// is ran from inside the application
define('IN_APP', true);

// Define a short-hand for directory separator
define('DS', DIRECTORY_SEPARATOR);

// Set the base path (this folder)
define('BASE', dirname($_SERVER['SCRIPT_FILENAME']));

// Locate folders
define('SYSTEM', BASE.'/system');
define('APP', BASE.'/app');

// Load error-function
require_once SYSTEM.'/error.php';

// Load Doctrine DBAL (database abstraction layer) for
// working with databases
$doctrine_path = SYSTEM.'/vendor/DoctrineDBAL-2.3.3';
require_once $doctrine_path.'/Doctrine/Common/ClassLoader.php';
$classLoader = new Doctrine\Common\ClassLoader('Doctrine', $doctrine_path);
$classLoader->register();

// Load helper-functions
require_once SYSTEM.'/helpers/language.php';
require_once SYSTEM.'/helpers/file.php';
require_once SYSTEM.'/helpers/url.php';

// Load class-aliases
// Aliases simplify the syntax by providing a "static" way of calling class instances
require_once SYSTEM.'/aliases.php';

// Get request, or use default (which is 'dashboard')
$request_string = Input::get('_request') != ''
	? trim(Input::get('_request'), '/')
	: 'dashboard';

// Store details about request for later use
Registry::set('request', $request_string);
Registry::set('segments', explode('/', $request_string));

{
	
	if(Auth::check() === false) {
		
		Route::register('login', function() {
			return Load::view('auth/login');
		});
		
		Route::register('login/action', function() {
			redirect(
				Auth::attempt(Input::post('username'), Input::post('password'))
					? ( Input::get('redirect') == '' ? 'dashboard' : Input::get('redirect') )
					: 'login?redirect='.( Input::get('redirect') == '' ? 'dashboard' : Input::get('redirect') )
			);
		});
		
		Route::register('(.*)', function($request) {
			redirect('login?redirect='.$request);
		});
		
	} else {
		
		Route::register('logout', function() {
			Auth::logout();
			redirect('login');
		});
		
		Route::register('dashboard', function() {
			return Load::controller('Dashboard')->index();
		});
		
		Route::register('section/([A-Za-z0-9_\-]+)(/.*|)', function($id, $other) {
			Registry::set('section', Load::section($id));
			return Load::controller('Section')->restful($other);
		});
		
	}
	
}

if($results = Route::run($request_string)) {
	echo $results;
} else {
	echo Load::view('errors/404');
}