Abundant
========

Abundant is a PHP-based tool that creates an admin interface for virtually any database-driven application. The process is simple:

1. Upload Abundant on a server (PHP5 and .htaccess required)
2. Edit some arrays inside some config-files.
3. Access with your browser. Voila!

On top of the default functionality Abundant is built to be very extensible. Meaning, for example, it's possible to integrate the userbase from your application.

Abundant uses Doctrine DBAL, and therefore supports any type of database Doctrine supports. These include MySQL, SQLite, PostreSQL, and so on...

> **NOTE!**  
> Abundant is under development. Bugs may occur. You carry the responsibility over your data when using Abundant.

Here's a couple of screenshots to illustrate what Abundant does. The layout uses Twitter Bootstrap for now, but it will get its own proper design at some point.

![Dashboard with one module](http://f.cl.ly/items/090p1n3m3E362P3N3V1G/Kuvankaappaus%202013-4-6%20kello%2022.15.26%20.png)
![Browsing a section](http://f.cl.ly/items/0l2n2b3L203B3D1c3e44/Kuvankaappaus%202013-4-6%20kello%2022.16.07%20.png)
![Editing a row](http://f.cl.ly/items/2s213V2V1y0Q1d122B3M/Kuvankaappaus%202013-4-6%20kello%2022.16.29%20.png)



Contents
--------

The [wiki](https://github.com/codeclown/Abundant/wiki) contains more in-depth details.

- [Getting started](#getting-started)
- [Default usage](#default-usage)
- [Types](#types)
  - [Creating types](#creating-types)
- [Authentication](#authentication)
- [Dashboard (modules)](#dashboard-modules)
- [License](#license)
- [Upcoming features](#upcoming-features)
- [Changelog](#changelog)
- [Credits](#credits)


Getting started
---------------

You should be ready to go in under 5 minutes. First upload Abundant to your server. As mentioned, PHP5 and .htaccess are required. Here's the `.htaccess`-file to be put inside the installation folder, should you have to manually add it:

```.htaccess
<IfModule mod_rewrite.c>
	
	Options +FollowSymLinks
	RewriteEngine On
	RewriteCond %{REQUEST_FILENAME} !-f
	RewriteCond %{REQUEST_FILENAME} !-d
	
	RewriteRule ^(.*)$ index.php?request=$1 [QSA]
	
</IfModule>
```

The first thing is to set up a database-connection. Edit `app/config/database.php` with your details. See [Doctrine DBAL manual](http://docs.doctrine-project.org/projects/doctrine-dbal/en/latest/reference/configuration.html) for what information is needed. Add the correct details to `app/config/database.php`. An example:

```php
<?php

return array(
	'driver' => 'pdo_mysql',
	'host' => 'localhost',
	'user' => '',
	'password' => '',
	'dbname' => 'blog',
);

?>
```

Then set up the authentication method (explained below).

Once everything is set, you can start defining the sections of your app.


Default usage
-------------

The way Abundant knows the structure of your database is by files containing arrays. Basically the only thing to do to get started (apart from setting up a way of authentication, see below) is to configure "sections". Sections are parts of your system, such as Posts and Users, that resemble a table in your database.

I'm going to guide you through setting up an imaginary blog. Start by creating a file in `app/sections` called `users.php`. Here is the code to put in that file:

```php
<?php

return array(
	
	// General information about this section
	'info' => array(
		
		// Title of this section
		'title' => 'Users',
		
		// Name of table in database
		'table' => 'users'
		
	),
	
	'fields' => array(
		
		// These are the "fields" in the database. They
		// are the same as columns in the table. Not all
		// columns from the table need to be included,
		// only the ones needed.
		
		// Note: ID (id, int, auto-increment, unique key)
		// is always included. If your tables don't have
		// such a column, Abundant won't work. Having a
		// column like this is good practise.
		
		// Fields are added as arrays, see below.
		
		array(
			
			// Unique ID of the field. In fact, this is the
			// actual name of the column in the table
			'id' => 'real_name',
			
			// A friendly title to show in the interface.
			'title' => 'Name',
			
			// Type of the field. We'll get to types below.
			'type' => 'text'
			
		),
		
		array(
			'id' => 'email',
			'title' => 'E-mail address',
			'type' => 'text',
		)
		
	),
	
);

?>
```

This array tells Abundant about a section called Users. The section has two fields, Name and E-mail address. Both fields are of type `text`.

There are other pieces of information we can tell Abundant. One of them is `link`. Add this to info-array:

```php
<?php

'link' => 'real_name'

?>
```

This information will be used when browsing the section. It tells Abundant what field should act as a link for the row. Now clicking the name of an user will show the editing-interface for that user.


Types
-----

Above I've mentioned "types". Types are what makes Abundant such a great tool for making a graphical interface for your database.

The two fields above were of type `text`. There are, however, several types to use. By default Abundant includes following types:

- text
- textbox
- select
- checkbox
- connect

Different types need different information. Below is some sample code for reference. These could be found from a section that manages blog-posts.

```php
<?php

array(
	
	'id' => 'content',
	'title' => 'Post content',
	
	'type' => 'textbox'
	
)

?>
```

```php
<?php

array(
	
	'id' => 'state',
	'title' => 'State',
	
	'type' => 'select',
	
	'options' => array(
		
		// 'draft' is what is saved to database.
		// 'Draft' is what is shown in a dropdown.
		'draft' => 'Draft',
		
		'private' => 'Private',
		'public' => 'Published'
		
	),
	
)

?>
```

```php
<?php

array(
	
	'id' => 'allow_comments',
	'title' => 'Allow comments',
	
	'type' => 'checkbox'
	
)

?>
```

```php
<?php

array(
	
	'id' => 'user_id',
	'title' => 'Author',
	
	'type' => 'connect',
	
	// Connect is a fabulous type. It allows
	// Abundant to show rows from another section.
	// In this example the user can choose the
	// post author from another section. The
	// available rows (users) are shown as a
	// dropdown to choose from.
	
	'connect' => array(
		
		// Section ID
		'section' => 'users',
		
		// Which column to display in dropdown
		'display' => 'real_name',
		
		// What to actually save to database. In
		// this case it's the ID of the user.
		'value' => 'id'
		
	)
	
)

?>
```

### Creating types

The default types work great with basic applications. However, you can build your own types by creating a PHP-class, and that's it. Take a look at the existing types to see how to create your own.


Authentication
--------------

By default Abundant uses a file-based authentication system, similar to how sections are configured. In this system users are added to a PHP-array in file `app/config/auth.php`.

For more advanced solutions, you can always create your own Auth-driver.


Dashboard (modules)
-------------------

When a user logs in to Abundant, a dashboard is shown. The dashboard consists of "modules", that are widgets of some sort. They are meant to provide a way to customize the admin experience to be more application-specific.

For example, in a blog-application you could show the posts that are marked as drafts, to remind the user to finish them. Create a file called `drafts.php` into `app/modules` and then edit the dashboard-configuration in `app/config/dashboard.php`:

```php
<?php

return array(
	
	// Each module is added as an array
	
	array(
		
		// Module ID. 'drafts' will look
		// for 'drafts.php'.
		'module' => 'drafts',
		
		// A title to show in the interface
		'title' => 'Drafts'
		
	)
	
);

?>
```

Next add some code to the actual module-file:

```php
<p>
	Be sure to finish these drafts!
</p>

<ul>
	
	<?php
		$section = Load::section('posts');
		$rows = $section->get_rows('state', 'draft');
	?>
	
	<?php while($row = $rows->fetch()): ?>
		<li>
			<a href="<?php echo url($section->url.'/edit/'.$row['id']); ?>">
				<?php echo $row['title']; ?>
			</a>
		</li>
	<?php endwhile; ?>
	
</ul>
```


License
-------

Abundant is licensed under MIT:

> Copyright (C) 2013 Martti Laine (@codeclown)
> 
> Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
> 
> The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
> 
> THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Meaning, you can use it for any purpose, whatsoever, as long as you keep this same license notice (license.txt) intact. Go crazy!


Upcoming features
-----------------

- Support for multiple themes.
- A new design.
- Full documentation (and site).


Changelog
---------

- 0.1 Initial release


Credits
-------
Created by Martti Laine ([@codeclown](http://twitter.com/codeclown)). Original idea from [PureEdit](http://pureedit.com).