<?php

/**
 * AliasLoader
 * 
 * Holds instances of classes. Forwards static calls on
 * so-called alias-classes to the actual instance.
 * 
 * So, when `Class::method()` is called, an instance of
 * `Class` is either created or retrieved from memory
 * and its method is called.
 */
class AliasLoader {
	
	/**
	 * Holds the instances of classes (meaning, cache)
	 *
	 * @var array
	 */
	public static $instances = array();
	
	/**
	 * Forwards static calls of a class to the methods of the actual instance
	 *
	 * @param string $method
	 * @param array $arguments
	 *
	 * @return mixed
	 */
	public static function __callStatic($method, $arguments) {
		
		// What class was called statically
		$class_name = get_called_class();
		
		// If an instance of this class is already stored, use it
		if(isset(static::$instances[$class_name])) {
			$instance = static::$instances[$class_name];
		} else {
			
			require_once SYSTEM.'/classes/'.strtolower($class_name).'.php';
			
			// The actual class resides under a different namespace (='Classes')
			$real_class_name = 'Classes\\'.$class_name;
			
			// Create an instance, and store it for future use
			$instance = new $real_class_name;
			static::$instances[$class_name] = $instance;
			
		}
		
		return call_user_func_array(array($instance, $method), $arguments);
		
	}
	
}

class Input extends AliasLoader {}
class Registry extends AliasLoader {}
class Load extends AliasLoader {}
class Route extends AliasLoader {}
class Config extends AliasLoader {}
class Auth extends AliasLoader {}
class Language extends AliasLoader {}
class Message extends AliasLoader {}