<?php

namespace Classes;

/**
 * Message
 * 
 * Messages are simple notifications that are shown
 * after an action. This class stores messages and
 * shows them on the next page. Messages are shown
 * once only, then deleted.
 */
class Message {
	
	/**
	 * Simply create the session-variable in case it hasn't been initialised yet
	 */
	public function __construct() {
		if(!isset($_SESSION['messages'])) $_SESSION['messages'] = array();
	}
	
	/**
	 * Store a message
	 * 
	 * The name 'show' fits better in the code :)
	 *
	 * @param string $color
	 * @param string $content
	 */
	public function show($color, $content) {
		$_SESSION['messages'][] = array($color, $content);
	}
	
	/**
	 * Print the messages
	 * 
	 * This is called in HTML.
	 */
	public function print_messages() {
		
		$html = '';
		foreach($_SESSION['messages'] as $message) {
			$html .= \Load::view('templates/message')->with(array(
				'color' => $message[0],
				'content' => $message[1]
			));
		}
		echo $html;
		
		// Clear the messages so they won't be shown again
		$_SESSION['messages'] = array();
		
	}
	
}