<?php

namespace Classes;

class Route {
	
	/**
	 * Holds the callbacks
	 *
	 * @var array
	 */
	private $_callbacks = array();
	
	/**
	 * Called after registering all routes
	 * 
	 * Parses all routes and checks if they match to request.
	 * 
	 * Returns false if no route matches.
	 *
	 * @param string $request
	 * @return boolean
	 */
	public function run($request) {
		
		foreach($this->_callbacks as $url => $callback) {
			
			// Turn URL into a valid regex
			// e.g. 'section/([0-9]+)' => '/section\/([0-9]+)/'
			$regex = '/^'.preg_replace('/\//', '\/', $url).'$/';
			
			if(preg_match($regex, $request, $matches)) {
				
				// First item in the array is the whole string, from
				// there on begin the arguments
				$arguments = array();
				for($i = 1; $i < count($matches); $i++) $arguments[] = $matches[$i];
				
				return call_user_func_array($callback, $arguments);
				
			}
			
		}
		
		// We got this far, it's a 404
		return false;
		
	}
	
	/**
	 * Register a callback for a route
	 *
	 * @param string $url
	 * @param function $callback
	 */
	public function register($url, $callback) {
		$this->_callbacks[$url] = $callback;
	}
	
}