<?php

namespace Controllers;
use \Skeletons\Controller;

class Section extends Controller {
	
	public function __construct() {
		$this->section = \Registry::get('section');
	}
	
	/**
	 * Redirect to browse
	 */
	public function index() {
		redirect($this->section->url.'/browse');
	}
	
	/**
	 * Browse rows
	 *
	 * Show table of rows, and associated tools.
	 */
	public function get_browse() {
		
		$section = $this->section;
		
		$page = \Input::get('page') !== false ? (int) \Input::get('page') : 1;
		$per_page = \Config::get('general', 'per_page');
		
		$total = $section->db->executeQuery('
			SELECT COUNT(*) as `count` FROM `'.$section->table.'`
		')->fetch();
		$total = $total['count'];
		if($total == 0) $total = 1; // There's always one page, even without results :P
		$pages = array(
			'current' => $page,
			'total' => ceil($total / $per_page)
		);
		
		$rows = $section->db->executeQuery('
			SELECT '.$section->fields_to_select.' FROM `'.$section->table.'`
			ORDER BY `id`
			LIMIT '.( ($page - 1) * $per_page ).', '.$per_page.'
		')->fetchAll();
		
		$data = array(
			'section' => $section,
			'rows' => $rows,
			'pages' => $pages
		);
		
		return \Load::view('section/browse')->with($data);
		
	}
	
	/**
	 * Add a new row
	 * 
	 * Show form-page with empty inputs.
	 */
	public function get_add() {
		return $this->_form_page();
	}
	
	/**
	 * Edit an existing row
	 * 
	 * Show form-page with prefilled inputs.
	 *
	 * @param int $id
	 */
	public function get_edit($id) {
		$section = $this->section;
		$row = $section->get_row('id', $id);
		return $this->_form_page($row);
	}
	
	/**
	 * Show search-form
	 * 
	 * The form's action will point to 'browse'
	 */
	public function get_search() {
		$section = $this->section;
		return \Load::view('section/search')->with('section', $section);
	}
	
	private function _form_page($row = false) {
		
		$section = $this->section;
		
		return \Load::view('section/form')->with(array(
			'section' => $section,
			'row' => $row
		));
		
	}
	
	/**
	 * Save row-details to database
	 * 
	 * If an ID is given, edit the row with that ID.
	 * Otherwise, add a new row.
	 *
	 * @param boolean|int $id
	 */
	public function post_save($id = false) {
		
		$section = $this->section;
		
		$arguments = array();
		foreach($section->fields as $field) {
			$arguments[$field['id']] = \Load::type($field['type'])->filter(\Input::post($field['id']));
		}
		
		// Next we either add a new row or edit an existing one
		
		if($id === false) {
			$section->db->insert($section->table, $arguments);
		} else {
			$row = $section->get_row('id', $id);
			$section->db->update($section->table, $arguments, array('id' => $row['id']));
		}
		
		\Message::show('blue', __($id === false ? 'Row added!' : 'Changes saved!'));
		redirect($section->url);
		
	}
	
	/**
	 * Delete rows
	 * 
	 * An array of ID's to delete is provided with checkboxes.
	 */
	public function post_delete() {
		$section = $this->section;
		$section->delete_rows(\Input::post('delete'));
		\Message::show('blue', __('Rows deleted!'));
		redirect($section->url);
	}
	
}