<?php

namespace Skeletons;

class Section {
	
	/**
	 * Load configuration
	 * 
	 * Acts pretty much like a constructor: gets things rolling.
	 *
	 * @param string $id
	 * @param array $config
	 */
	public function load_configuration($id, $config) {
		
		$this->_config = $config;
		
		if(!is_array($config)) error('Section `'.$id.'` does not return an array.');
		
		$required_fields = array(
			'info' => array('title', 'table'),
			'fields' => array()
		);
		
		foreach($required_fields as $main => $sub) {
			if(!isset($config[$main])) error('Section `'.$id.'` does not have the ['.$main.']-field.');
			foreach($sub as $key) {
				if(!isset($config[$main][$key])) error('Section `'.$id.'` does not have the ['.$main.']['.$key.']-field.');
			}
		}
		
		// Store some commonly used values for easier access
		$this->id = $id;
		$this->title = $config['info']['title'];
		$this->table = $config['info']['table'];
		$this->fields = $config['fields'];
		$this->link = isset($config['info']['link']) ? $config['info']['link'] : 'id';
		
		$this->url = 'section/'.$this->id;
		
		$this->fields_to_select = '`id`';
		foreach($this->_config['fields'] as $field) $this->fields_to_select .= ', `'.$field['id'].'`';
		
		$this->setup_db();
		
	}
	
	/**
	 * Setup database-connection
	 */
	public function setup_db() {
		
		$parameters = \Config::get('database');
		
		// Override any configuration with section-specific values, if present
		if(isset($this->_config['connection']) && is_array($this->_config['connection'])) {
			foreach($this->_config['connection'] as $key => $value) {
				$parameters[$key] = $value;
			}
		}
		
		$config = new \Doctrine\DBAL\Configuration();
		$this->db = \Doctrine\DBAL\DriverManager::getConnection($parameters, $config);
		
	}
	
	/**
	 * Get rows
	 *
	 * Note! $key isn't escaped.
	 *
	 * @param string $key
	 * @param mixed $value
	 */
	public function get_rows($key, $value) {
		return $this->db->executeQuery('
			SELECT '.$this->fields_to_select.' FROM `'.$this->table.'`
			WHERE `'.$key.'` = ?
		', array($value));
	}
	
	/**
	 * Get a single row
	 *
	 * @see $this->get_rows()
	 */
	public function get_row($key, $value) {
		return $this->get_rows($key, $value)->fetch();
	}
	
	/**
	 * Delete rows by ID
	 *
	 * @param array|int $ids
	 */
	public function delete_rows($ids) {
		
		if(!is_array($ids)) $ids = array($ids);
		
		$this->db->executeQuery('
			DELETE  FROM `'.$this->table.'`
			WHERE `id` IN (?)
		', array($ids), array(\Doctrine\DBAL\Connection::PARAM_INT_ARRAY));
		
	}
	
}