<?php

namespace Skeletons;

class View {
	
	/**
	 * Path to view-file
	 *
	 * @var string
	 */
	public $path = '';
	
	/**
	 * Holds the variables that will be passed to the view
	 *
	 * @var array
	 */
	private $_variables = array();
	
	/**
	 * Set variable(s) to be available for view in HTML
	 *
	 * Variables can be given as an array, for example:
	 *     echo Load::view('foo')->with(array(
	 *         'first' => 'Hey',
	 *         'second' => 'Is it me you\'re looking for?'
	 *     ));
	 * 
	 * Also a single variable can be set:
	 *     echo Load::view('foo')->with('title', 'Lionel Richie');
	 *
	 * @param string|array $data
	 * @param mixed $value
	 */
	public function with($data, $value = false) {
		
		// If $data isn't an array, it's a key for $value
		if(!is_array($data) && $value !== false) $data = array($data => $value);
		
		// Save each variable for use inside HTML
		foreach($data as $key => $value) $this->_variables[$key] = $value;
		
		return $this;
		
	}
	
	/**
	 * Return the view as HTML
	 * 
	 * Print a view:
	 *     echo Load::view('foo');
	 *
	 * @return string
	 */
	public function __toString() {
		
		extract($this->_variables);
		
		ob_start();
		require_once $this->path;
		return ob_get_clean();
		
	}
	
}